from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, RESPONSE_CANCEL, \
	STOCK_OK, RESPONSE_OK, Label, Entry, WRAP_WORD, ScrolledWindow, SHADOW_IN, POLICY_NEVER, \
	POLICY_AUTOMATIC, SHADOW_NONE, Table, FILL, RadioButton
from gtksourceview import SourceLanguagesManager, SourceBuffer, SourceView
from pango import FontDescription
from gobject import timeout_add, source_remove

from PreviewArea import PreviewArea
from common import Snippet, TEXT, MATH

PREVIEW_DELAY = 1000		## timeout in ms after which the preview is rendered

class SnippetDialog (Dialog):
	def __init__(self):
		Dialog.__init__(self, "", None, DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.snippet = None
		self.render_timer = None
		
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		
		self.entry_name = Entry()
		
		lbl_name = Label(_("_Name:"))
		lbl_name.set_alignment(0, .5)
		lbl_name.set_use_underline(True)
		lbl_name.set_mnemonic_widget(self.entry_name)
		
		language = SourceLanguagesManager().get_language_from_mime_type("text/x-tex")
		self.buf_definition = SourceBuffer()
		self.buf_definition.set_language(language)
		self.buf_definition.set_highlight(True)
		self.buf_definition.connect("changed", self.__on_definition_changed)
		
		view_definition = SourceView(self.buf_definition)
		view_definition.set_wrap_mode(WRAP_WORD)
		view_definition.modify_font(FontDescription("Monospace"))
		
		scr_definition = ScrolledWindow()
		scr_definition.set_policy(POLICY_NEVER, POLICY_AUTOMATIC);
		scr_definition.set_shadow_type(SHADOW_IN)
		scr_definition.add(view_definition)
		
		lbl_definition = Label(_("_Definition:"))
		lbl_definition.set_alignment(0, 0)
		lbl_definition.set_use_underline(True)
		lbl_definition.set_mnemonic_widget(view_definition)
		
		self.preview = PreviewArea()	
		scr_preview = ScrolledWindow()
		scr_preview.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		scr_preview.set_shadow_type(SHADOW_NONE)
		scr_preview.add_with_viewport(self.preview)
		#scr_preview.connect("button-press-event", lambda *a: self.renderPreview())
		
		lbl_scope = Label(_("Scope:"))
		lbl_scope.set_alignment(0, 0)
		
		self.radio_text = RadioButton(None, _("_Text"))
		self.radio_math = RadioButton(self.radio_text, _("_Mathematical"))
		self.radio_both = RadioButton(self.radio_text, _("_Both"))
		
		table = Table()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		
		table.attach(lbl_name, 0, 1, 0, 1, FILL, FILL)
		table.attach(self.entry_name, 1, 4, 0, 1, FILL, FILL)
		table.attach(lbl_definition, 0, 1, 1, 2, FILL)
		table.attach(scr_definition, 1, 4, 1, 2)
		table.attach(scr_preview, 1, 4, 2, 3)
		table.attach(lbl_scope, 0, 1, 3, 4, FILL, FILL)
		table.attach(self.radio_text, 1, 2, 3, 4, FILL, FILL)
		table.attach(self.radio_math, 2, 3, 3, 4, FILL, FILL)
		table.attach(self.radio_both, 3, 4, 3, 4, FILL, FILL)
		
		self.vbox.add(table)
	
	def __on_definition_changed(self, sourcebuffer):
		if self.render_timer:
			source_remove(self.render_timer)
		self.render_timer = timeout_add(PREVIEW_DELAY, self.__render_preview)
	
	def __render_preview(self):
		""" This is called after a timeout """
		source = self.buf_definition.get_text(self.buf_definition.get_start_iter(), self.buf_definition.get_end_iter())
		
		if self.radio_math.get_active():
			source = "$%s$" % source
		
		self.preview.set_source(source)
		self.preview.render()
		self.render_timer = None
		
		return False
	
	def run(self, snippet=Snippet()):
		""" Load the given Snippet and run this dialog """
		self.snippet = snippet
		
		self.entry_name.set_text(snippet.name)
		self.buf_definition.set_text(snippet.definition)
		if snippet.scope == TEXT:
			self.radio_text.set_active(True)
		elif snippet.scope == MATH:
			self.radio_math.set_active(True)
		elif snippet.scope == TEXT|MATH:
			self.radio_both.set_active(True)
		
		if snippet.key:
			self.set_title(_("Edit Snippet ") + snippet.name)
		else:
			self.set_title(_("New Snippet "))
		
		self.show_all()
		
		return Dialog.run(self)
	
	def get_snippet(self):
		""" Return the Snippet resulting from this dialog """
		
		self.snippet.name = self.entry_name.get_text()
		self.snippet.definition = self.buf_definition.get_text(self.buf_definition.get_start_iter(), self.buf_definition.get_end_iter())
		if self.radio_text.get_active():
			self.snippet.scope = TEXT
		elif self.radio_math.get_active():
			self.snippet.scope = MATH
		elif self.radio_both.get_active():
			self.snippet.scope = BOTH
		
		return self.snippet
		
