from gtk import ScrolledWindow, ListStore, TreeView, CellRendererText, POLICY_AUTOMATIC, SHADOW_IN, \
	Toolbar, ToolButton, STOCK_ADD, VBox, Tooltips, STOCK_REMOVE, STOCK_EDIT, RESPONSE_OK
from common import ConditionalSource

class SnippetsPanel (VBox):
	snippet_dialog = None

	def __init__(self, instance):
		VBox.__init__(self)
		
		self.instance = instance
		
		#
		# Toolbar
		#
		self.tooltips = Tooltips()
		
		toolbar = Toolbar()
		
		button_add = ToolButton(STOCK_ADD)
		button_add.set_tooltip(self.tooltips, _("Add a snippet..."))
		button_add.connect("clicked", self.__on_add_snippet)
		toolbar.insert(button_add, -1)
		
		button_remove = ToolButton(STOCK_REMOVE)
		button_remove.set_tooltip(self.tooltips, _("Remove selected snippet"))
		button_remove.connect("clicked", self.__on_remove_snippet)
		toolbar.insert(button_remove, -1)
		
		button_edit = ToolButton(STOCK_EDIT)
		button_edit.set_tooltip(self.tooltips, _("Edit selected snippet..."))
		button_edit.connect("clicked", self.__on_edit_snippet)
		toolbar.insert(button_edit, -1)
		
		self.pack_start(toolbar, False)
		
		#
		# List
		#
		self.model_snippets = ListStore(str, object)	## label markup, Snippet object
		
		self.view_snippets = TreeView(self.model_snippets)
		self.view_snippets.set_headers_visible(False)
		self.view_snippets.insert_column_with_attributes(-1, "", CellRendererText(), markup=0)
		self.view_snippets.connect("row-activated", self.__on_snippet_activated)
		
		scrolled = ScrolledWindow()
		scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
		scrolled.set_shadow_type(SHADOW_IN)
		
		scrolled.add(self.view_snippets)
		
		self.pack_start(scrolled, True)
		
		self.show_all()
		
		self.instance.settings.monitor_snippets(lambda *x: self.__load_snippets())
		self.__load_snippets()
		
	def __load_snippets(self):
		self.model_snippets.clear()
		for snippet in self.instance.settings.list_snippets():
			label = "%s\n<small><tt>%s</tt></small>" % (snippet.name, snippet.definition)
			self.model_snippets.append([label, snippet])
	
	def __on_snippet_activated(self, treeview, path, column):
		model = treeview.get_model()
		snippet = model.get_value(model.get_iter(path), 1)
		
		## TODO: DocumentWrapper needs to handle Snippet objects in insert_source()
		source = ConditionalSource(snippet.definition, snippet.packages)
		self.instance.get_active_wrapper().insert_source(source)
	
	def __on_add_snippet(self, toolbutton):
		if not self.snippet_dialog:
			from SnippetDialog import SnippetDialog
			self.snippet_dialog = SnippetDialog()
		
		if self.snippet_dialog.run() == RESPONSE_OK:
			new_snippet = self.snippet_dialog.get_snippet()
			self.instance.settings.save_snippet(new_snippet)
			
		self.snippet_dialog.hide()
		
	def __on_remove_snippet(self, toolbutton):
		store, it = self.view_snippets.get_selection().get_selected()
		if not it:
			return
			
		snippet = store.get_value(it, 1)
		self.instance.settings.delete_snippet(snippet)
	
	def __on_edit_snippet(self, toolbutton):
		store, it = self.view_snippets.get_selection().get_selected()
		if not it:
			return
			
		snippet = store.get_value(it, 1)
		
		if not self.snippet_dialog:
			from SnippetDialog import SnippetDialog
			self.snippet_dialog = SnippetDialog()
		
		if self.snippet_dialog.run(snippet) == RESPONSE_OK:
			self.instance.settings.save_snippet(self.snippet_dialog.get_snippet())
		
		self.snippet_dialog.hide()
		
