from gtk import ScrolledWindow, POLICY_AUTOMATIC, POLICY_NEVER, SHADOW_IN, VBox, ListStore, IconView, SELECTION_SINGLE, Layout, \
	VPaned
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from common import PLUGIN_PATH, MATH

symbols = {
	"operators" : ("approx","ast","bullet","cap","cdot","circ","cong","cup","div","equiv","eth","exists","forall","geq","gg","in","land","leq","ll","lor","mid","nabla","neg","neq","nexists","ni","parallel","perp","partial","pm","propto","setminus","sim","simeq","star","subset","subseteq","supset","supseteq","times","uplus","varnothing"),
	"arrows" 	: ("Leftarrow","Leftrightarrow","Longleftarrow","Longleftrightarrow","Longrightarrow","Rightarrow","hookleftarrow","hookrightarrow","leadsto","leftarrow","leftharpoondown","leftharpoonup","leftrightarrow","leftrightarrows","leftrightharpoons","longleftarrow","longleftrightarrow","longmapsto","longrightarrow","mapsto","rightarrow","rightharpoondown","rightharpoonup","rightleftarrows","rightleftharpoons"),
	"limiters" 	: ("lbrace","rbrace","langle","rangle","lfloor","rfloor","lceil","rceil","vert","Vert"),
	"greek" 	: ("alpha","beta","gamma","delta","epsilon","varepsilon","zeta","eta","theta","vartheta","iota","kappa","lambda","mu","nu","xi","pi","varpi","rho","varrho","sigma","varsigma","tau","upsilon","phi","varphi","chi","psi","omega","Gamma","Delta","Theta","Lambda","Xi","Pi","Sigma","Upsilon","Phi","Psi","Omega"),
	"special" 	: ("ldots","cdots","vdots","ddots","Im","Re","complement","hbar","infty","S","textcopyright","textregistered","texttrademark","P","dagger","wp","aleph","angle","prime","ell","sum","prod","int","iint","oint")
}

class SymbolsPanel (VBox):
	def __init__(self, instance):
		VBox.__init__(self)
		
		self.instance = instance
		
		paned = VPaned()
		self.pack_start(paned, True)
		
		for (page, names) in symbols.items():
			model = ListStore(Pixbuf)
			
			for name in names:
				model.append([pixbuf_new_from_file("%s/pixmaps/%s/%s.png" % (PLUGIN_PATH, page, name))])
			
			view = IconView(model)
			view.set_pixbuf_column(0)
			view.set_selection_mode(SELECTION_SINGLE)
			view.connect("selection-changed", self.__on_symbol_selected, page)
			view.set_item_width(-1)			# use largest item width
			view.set_spacing(0)
			view.set_column_spacing(0)
			view.set_row_spacing(0)
			view.set_columns(-1)			# show as much as possible
			view.set_text_column(-1)
			
			scr = ScrolledWindow()
			scr.set_policy(POLICY_NEVER, POLICY_AUTOMATIC)
			scr.add(view)
			
			## cascade paned widgets
			paned.pack1(scr, False, False)
			n_paned = VPaned()
			paned.pack2(n_paned, False, False)
			paned = n_paned
			
		self.show_all()
	
	def __on_symbol_selected(self, icon_view, page):
		try: 
			index = icon_view.get_selected_items()[0][0]
		except IndexError: 
			return				# must be caught after unselect_all()
		
		self.instance.get_active_wrapper().insert_source({MATH : "\\" + symbols[page][index]})
		
		icon_view.unselect_all()
	
