from os.path import expanduser

PLUGIN_PATH = expanduser("~/.gnome2/gedit/plugins/LaTeXPlugin")

ALT_COLOR = "#707070"

LABEL, BIBITEM = 1, 2

## environments
UNKNOWN, TEXT, MATH = 0, 1, 2

## image formats
PNG, JPG, GIF = 1, 2, 4

def escape_text(s):
	"""
	Escapes text so that the markup parser will parse it verbatim. Less than, 
	greater than, ampersand, etc. are replaced with the corresponding entities.
	"""
	return s.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")

class ConditionalSource (object):
	"""
	This encapsulates LaTeX source that requires certain packages
	"""
	def __init__(self, source, packages):
		self.source, self.packages = source, packages
	
	def get_source(self):
		return self.source
	
	def get_required_packages(self):
		return self.packages

class PrefaceSource (object):
	"""
	This encapsulates a source that is to be inserted into the document's
	preface section 
	
	This type of source may only contain a string 
	"""
	def __init__(self, source):
		self.source = str(source)
	
	def get_source(self):
		return self.source

class Profile:
	"""
	This encapsulates a custom build profile
	"""
	def __init__(self, name="MyProfile", commands=[], view_command="gnome-open", accel=""):
		self.key, self.name, self.commands, self.view_command, self.accel = None, name, commands, view_command, accel

class Snippet:
	def __init__(self, name="MySnippet", definition="", packages=[], scope=None):
		self.key, self.name, self.definition, self.packages, self.scope = None, name, definition, packages, scope
	
